#include <UbiConstants.h>
#include <UbiTypes.h>
#include <UbidotsEsp32Mqtt.h>
#include <Wire.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_GFX.h>
#include <Adafruit_NeoPixel.h>
/******************************************
 *
 * This example works for both Industrial and STEM users.
 *
 * Developed by Jose Garcia, https://github.com/jotathebest/
 *
 * ****************************************/

/****************************************
 * Include Libraries
 ****************************************/
#include "UbidotsEsp32Mqtt.h"
#include <DFRobot_MAX30102.h>

/*Variables, parametros para la pantalla OLED*/
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET -1

//Led pin and amount of leds i'm going to light up//
#define PIN 1       // Pin al que está conectado el DATA IN de la tira de LED
#define NUMPIXELS 12 // Número de NeoPixels en la tira

Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
DFRobot_MAX30102 particleSensor;

/****************************************
 * Define Constants
 ****************************************/
const char *UBIDOTS_TOKEN = "BBUS-sCCqtUIMw5p0pGasLTmwCKitHPuvJb";  // Put here your Ubidots TOKEN
const char *WIFI_SSID = "FMC";                                       // Put here your Wi-Fi SSID
const char *WIFI_PASS = "EGAfKfCjMC5.7geuLryZ";                        // Put here your Wi-Fi password
const char *DEVICE_LABEL = "esp32";                                 // Put here your Device label to which data  will be published
const char *ubi_spo2 = "spo2";                                      // Put here your Variable label to which data  will be published
const char *ubi_bpm = "bpm";                                        // Put here your Variable label to which data  will be published
const char *ubi_led = "led";                                        // Replace with your variable label to subscribe to

uint8_t analogPin = 34;  // Pin used to read data from GPIO34 ADC_CH6.

Ubidots ubidots(UBIDOTS_TOKEN);

/****************************************
 * Auxiliar Functions
 ****************************************/

void callback(char *topic, byte *payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payload[i]);

    if ((char)payload[0] == '1') {
      digitalWrite(LED_BUILTIN, HIGH);
    } else {
      digitalWrite(LED_BUILTIN, LOW);
    }
  }
  Serial.println();
}

/****************************************
 * Main Functions
 ****************************************/

void setup() {

  //Inicia los leds y se defina el color//
  pixels.begin();  // Inicializa la tira de LEDs
  setAllPixelsColor(pixels.Color(255, 0, 0)); // Establece todos los LEDs al color rojo

  // put your setup code here, to run once:
  Serial.begin(115200);

  //Aqui se inicia la pantalla//
 if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3c)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;)
      ;
  }
  delay(2000);

  //Tamaño de letra//
  display.clearDisplay();
  display.setTextSize(1,2);
  display.setTextColor(WHITE);

  display.setCursor(0, 0);
  // Display static text
  display.println("Monitoring");
  display.display(); 
  delay(100);


  // ubidots.setDebug(true);  // uncomment this to make debug messages available
  ubidots.connectToWifi(WIFI_SSID, WIFI_PASS);
  ubidots.setCallback(callback);
  ubidots.setup();
  ubidots.reconnect();
  ubidots.subscribeLastValue(DEVICE_LABEL, ubi_led);  // Insert the dataSource and Variable's Labels


  while (!particleSensor.begin()) {
    Serial.println("MAX30102 was not found");
    delay(1000);
  }

  particleSensor.sensorConfiguration(/*ledBrightness=*/50, /*sampleAverage=*/SAMPLEAVG_4,
                                     /*ledMode=*/MODE_MULTILED, /*sampleRate=*/SAMPLERATE_100,
                                     /*pulseWidth=*/PULSEWIDTH_411, /*adcRange=*/ADCRANGE_16384);
}

int32_t SPO2;           //SPO2
int8_t SPO2Valid;       //Flag to display if SPO2 calculation is valid
int32_t heartRate;      //Heart-rate
int8_t heartRateValid;  //Flag to display if heart-rate calculation is valid

void loop() {
  // put your main code here, to run repeatedly:
  if (!ubidots.connected()) {
    ubidots.reconnect();
  }
  Serial.println(F("Wait about four seconds"));
  particleSensor.heartrateAndOxygenSaturation(/**SPO2=*/&SPO2, /**SPO2Valid=*/&SPO2Valid, /**heartRate=*/&heartRate, /**heartRateValid=*/&heartRateValid);
  //Print result
  Serial.print(F("heartRate="));
  Serial.print(heartRate, DEC);

  Serial.print(F(", heartRateValid="));
  Serial.print(heartRateValid, DEC);
  Serial.print(F("; SPO2="));
  Serial.print(SPO2, DEC);
  Serial.print(F(", SPO2Valid="));
  Serial.println(SPO2Valid, DEC);

  //Impresión en pantalla//
  display.clearDisplay();
  display.setCursor(0, 0);
  display.println(F("HeartRate: "));
  display.setCursor(88, 0);
  display.println(heartRate);

  display.setCursor(0, 20);
  display.println(F("SPO2: "));
  display.setCursor(40, 20);
  display.println(SPO2);

  display.display();

  setAllPixelsColor(pixels.Color(255, 0, 0)); // Establece todos los LEDs al color rojo



  //float value = analogRead(analogPin);
  ubidots.add(ubi_spo2, SPO2);      // Insert your variable Labels and the value to be sent
  ubidots.add(ubi_bpm, heartRate);  // Insert your variable Labels and the value to be sent
  ubidots.publish(DEVICE_LABEL);

  ubidots.loop();
}

void setAllPixelsColor(uint32_t color) {
  for(int i = 0; i < NUMPIXELS; i++) {
    pixels.setPixelColor(i, color);  // Establece el color para cada LED
  }
  pixels.show();  // Muestra los cambios
}
